"use strict"; function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { newObj[key] = obj[key]; } } } newObj.default = obj; return newObj; } } function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }// src/node/cli.ts
var _chalk = require('chalk'); var _chalk2 = _interopRequireDefault(_chalk);
var _yargs = require('yargs'); var _yargs2 = _interopRequireDefault(_yargs);
var _helpers = require('yargs/helpers');

// src/node/build.ts
var _path = require('path');
var _fsextra = require('fs-extra'); var _fsextra2 = _interopRequireDefault(_fsextra);
var _vite = require('vite');

async function build(cliOptions = {}) {
  const mode = process.env.MODE || process.env.NODE_ENV || cliOptions.mode || "production";
  const config = await _vite.resolveConfig.call(void 0, {}, "build", mode);
  const cwd = process.cwd();
  const root = config.root || cwd;
  const ssgOut = _path.join.call(void 0, root, ".vite-ssg-temp");
  const {
    entry = await detectEntry(root),
    onAfterClientBuild
  } = Object.assign({}, config.ssgOptions || {}, cliOptions);
  if (_fsextra2.default.existsSync(ssgOut))
    await _fsextra2.default.remove(ssgOut);
  console.log(`
${_chalk2.default.gray("[vite-ssg]")} ${_chalk2.default.yellow("Build for client...")}`);
  await _vite.build.call(void 0, {
    build: {
      ssrManifest: true,
      rollupOptions: {
        input: {
          app: _path.join.call(void 0, root, "./index.html")
        }
      }
    },
    mode: config.mode
  });
  onAfterClientBuild == null ? void 0 : onAfterClientBuild();
  console.log(`
${_chalk2.default.gray("[vite-ssg]")} ${_chalk2.default.yellow("Build for server...")}`);
  process.env.VITE_SSG = "true";
  const ssrEntry = await resolveAlias(config, entry);
  await _vite.build.call(void 0, {
    build: {
      ssr: ssrEntry,
      outDir: ssgOut,
      minify: false,
      cssCodeSplit: false,
      rollupOptions: {
        output: {
          entryFileNames: "[name].mjs",
          format: "esm"
        }
      }
    },
    mode: config.mode
  });
}
async function detectEntry(root) {
  const scriptSrcReg = /<script(?:.*?)src=["'](.+?)["'](?!<)(?:.*)\>(?:[\n\r\s]*?)(?:<\/script>)/img;
  const html = await _fsextra2.default.readFile(_path.join.call(void 0, root, "index.html"), "utf-8");
  const scripts = [...html.matchAll(scriptSrcReg)];
  const [, entry] = scripts.find((matchResult) => {
    const [script] = matchResult;
    const [, scriptType] = script.match(/.*\stype=(?:'|")?([^>'"\s]+)/i) || [];
    return scriptType === "module";
  }) || [];
  return entry || "src/main.ts";
}
async function resolveAlias(config, entry) {
  const resolver = config.createResolver();
  const result = await resolver(entry, config.root);
  return result || _path.join.call(void 0, config.root, entry);
}

// src/node/render.ts

var _inspector = require('inspector'); var _inspector2 = _interopRequireDefault(_inspector);
var _os = require('os'); var _os2 = _interopRequireDefault(_os);



var _piscina = require('piscina'); var _piscina2 = _interopRequireDefault(_piscina);

// src/node/utils.ts
function routesToPaths(routes) {
  if (!routes)
    return ["/"];
  const paths = /* @__PURE__ */ new Set();
  const getPaths = (routes2, prefix = "") => {
    prefix = prefix.replace(/\/$/g, "");
    for (const route of routes2) {
      let path = route.path;
      if (route.path) {
        path = prefix && !route.path.startsWith("/") ? `${prefix}/${route.path}` : route.path;
        paths.add(path);
      }
      if (Array.isArray(route.children))
        getPaths(route.children, path);
    }
  };
  getPaths(routes);
  return [...paths];
}

// src/node/critical.ts
async function getCritters(outDir, options = {}) {
  try {
    const CrittersClass = (await Promise.resolve().then(() => _interopRequireWildcard(require("critters")))).default;
    const fs3 = await Promise.resolve().then(() => _interopRequireWildcard(require("fs")));
    class CrittersClassWithFS extends CrittersClass {
      constructor(...args) {
        super(...args);
        this.fs = fs3;
      }
    }
    return new CrittersClassWithFS({
      path: outDir,
      logLevel: "warn",
      external: true,
      inlineFonts: true,
      preloadFonts: true,
      ...options
    });
  } catch (e) {
    return void 0;
  }
}

// src/node/render.ts
function DefaultIncludedRoutes(paths) {
  return paths.filter((i) => !i.includes(":") && !i.includes("*"));
}
async function render(cliOptions = {}) {
  var _a;
  const mode = process.env.MODE || process.env.NODE_ENV || cliOptions.mode || "production";
  process.env.NODE_ENV = mode;
  const config = await _vite.resolveConfig.call(void 0, {}, "build", mode);
  const cwd = process.cwd();
  const root = config.root || cwd;
  const ssgOut = _path.join.call(void 0, root, ".vite-ssg-temp");
  const outDir = config.build.outDir || "dist";
  const out = _path.isAbsolute.call(void 0, outDir) ? outDir : _path.join.call(void 0, root, outDir);
  const {
    script = "sync",
    mock = false,
    entry = await detectEntry2(root),
    formatting = "none",
    crittersOptions = {},
    includedRoutes = DefaultIncludedRoutes,
    // onBeforePageRender,
    // onPageRendered,
    onFinished,
    dirStyle = "flat",
    includeAllRoutes = false
  } = Object.assign({}, config.ssgOptions || {}, cliOptions);
  const ssrEntry = await resolveAlias2(config, entry);
  const prefix = process.platform === "win32" ? "file://" : "";
  const ssrEntryPath = _path.join.call(void 0, prefix, ssgOut, `${_path.parse.call(void 0, ssrEntry).name}.mjs`);
  const { createApp } = await Promise.resolve().then(() => _interopRequireWildcard(require(ssrEntryPath)));
  const { routes } = await createApp(false);
  let routesPaths = includeAllRoutes ? routesToPaths(routes) : await includedRoutes(routesToPaths(routes));
  routesPaths = Array.from(new Set(routesPaths));
  const critters = crittersOptions !== false ? await getCritters(outDir, crittersOptions) : void 0;
  if (critters)
    console.log(`${_chalk2.default.gray("[vite-ssg]")} ${_chalk2.default.blue("Critical CSS generation enabled via `critters`")}`);
  if (mock) {
    const jsdomGlobal = (await Promise.resolve().then(() => _interopRequireWildcard(require("../jsdomGlobal-6UGIYNQH.js")))).default;
    jsdomGlobal();
  }
  const ssrManifest = JSON.parse(await _fsextra2.default.readFile(_path.join.call(void 0, out, "ssr-manifest.json"), "utf-8"));
  const indexHTML = await _fsextra2.default.readFile(_path.join.call(void 0, ssgOut, "index.html"), "utf-8").catch(async () => {
    let indexHTML2 = await _fsextra2.default.readFile(_path.join.call(void 0, out, "index.html"), "utf-8");
    indexHTML2 = rewriteScripts(indexHTML2, script);
    await _fsextra2.default.writeFile(_path.join.call(void 0, ssgOut, "index.html"), indexHTML2, "utf-8");
    return indexHTML2;
  });
  const maxThreads = _inspector2.default.url() ? 1 : Math.max(1, Math.floor(Math.min(_os2.default.cpus().length / 2, _os2.default.freemem() / (1.1 * 1024 ** 3))));
  console.log(`
${_chalk2.default.gray("[vite-ssg]")} ${_chalk2.default.yellow("Rendering")} ${_chalk2.default.blue(routesPaths.length)} ${_chalk2.default.yellow("pages...")} ${_chalk2.default.gray(`(${maxThreads} threads)`)}`);
  const pool = new (0, _piscina2.default)({
    filename: new URL("./renderWorker.mjs", import.meta.url).pathname,
    niceIncrement: 10,
    maxThreads
  });
  await Promise.all(
    routesPaths.map((route) => {
      return pool.run({
        route,
        dirStyle,
        outDir,
        indexHTML,
        formatting,
        out,
        ssrEntryPath,
        ssrManifest
      });
    })
  );
  const pwaPlugin = (_a = config.plugins.find((i) => i.name === "vite-plugin-pwa")) == null ? void 0 : _a.api;
  if (pwaPlugin == null ? void 0 : pwaPlugin.generateSW) {
    console.log(`
${_chalk2.default.gray("[vite-ssg]")} ${_chalk2.default.yellow("Regenerate PWA...")}`);
    await pwaPlugin.generateSW();
  }
  console.log(`
${_chalk2.default.gray("[vite-ssg]")} ${_chalk2.default.green("Build finished.")}`);
  await (onFinished == null ? void 0 : onFinished());
  const waitInSeconds = 15;
  const timeout = setTimeout(() => {
    console.log(`${_chalk2.default.gray("[vite-ssg]")} ${_chalk2.default.yellow(`Build process still running after ${waitInSeconds}s. There might be something misconfigured in your setup. Force exit.`)}`);
    process.exit(0);
  }, waitInSeconds * 1e3);
  timeout.unref();
}
function rewriteScripts(indexHTML, mode) {
  if (!mode || mode === "sync")
    return indexHTML;
  return indexHTML.replace(/<script type="module" /g, `<script type="module" ${mode} `);
}
async function detectEntry2(root) {
  const scriptSrcReg = /<script(?:.*?)src=["'](.+?)["'](?!<)(?:.*)\>(?:[\n\r\s]*?)(?:<\/script>)/img;
  const html = await _fsextra2.default.readFile(_path.join.call(void 0, root, "index.html"), "utf-8");
  const scripts = [...html.matchAll(scriptSrcReg)];
  const [, entry] = scripts.find((matchResult) => {
    const [script] = matchResult;
    const [, scriptType] = script.match(/.*\stype=(?:'|")?([^>'"\s]+)/i) || [];
    return scriptType === "module";
  }) || [];
  return entry || "src/main.ts";
}
async function resolveAlias2(config, entry) {
  const resolver = config.createResolver();
  const result = await resolver(entry, config.root);
  return result || _path.join.call(void 0, config.root, entry);
}

// src/node/cli.ts
_yargs2.default.call(void 0, _helpers.hideBin.call(void 0, process.argv)).scriptName("vite-ssg").usage("$0 [args]").command(
  "build",
  "Build SSG",
  (args) => args.option("script", {
    choices: ["sync", "async", "defer", "async defer"],
    describe: "Rewrites script loading timing"
  }).option("mock", {
    type: "boolean",
    describe: "Mock browser globals (window, document, etc.) for SSG"
  }),
  async (args) => {
    await build(args);
  }
).command(
  "render",
  "Render SSG",
  (args) => args,
  async (args) => {
    await render(args);
  }
).fail((msg, err, yargs2) => {
  console.error(`
${_chalk2.default.gray("[vite-ssg]")} ${_chalk2.default.red.bold("An internal error occurred.")}`);
  console.error(`${_chalk2.default.gray("[vite-ssg]")} ${_chalk2.default.white(`Please report an issue, if none already exists: ${_chalk2.default.underline("https://github.com/antfu/vite-ssg/issues")}`)}`);
  yargs2.exit(1, err);
}).showHelpOnFail(false).help().argv;
