"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { newObj[key] = obj[key]; } } } newObj.default = obj; return newObj; } } function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }// src/node/renderWorker.ts
var _path = require('path');
var _chalk = require('chalk'); var _chalk2 = _interopRequireDefault(_chalk);
var _serverrenderer = require('vue/server-renderer');
var _jsdom = require('jsdom');
var _fsextra = require('fs-extra'); var _fsextra2 = _interopRequireDefault(_fsextra);

// src/node/preload-links.ts
function renderPreloadLinks(document, modules, ssrManifest) {
  const seen = /* @__PURE__ */ new Set();
  const preloadLinks = [];
  Array.from(modules).forEach((id) => {
    const files = ssrManifest[id] || [];
    files.forEach((file) => {
      if (!preloadLinks.includes(file))
        preloadLinks.push(file);
    });
  });
  if (preloadLinks) {
    preloadLinks.forEach((file) => {
      if (!seen.has(file)) {
        seen.add(file);
        renderPreloadLink(document, file);
      }
    });
  }
}
function renderPreloadLink(document, file) {
  if (file.endsWith(".js")) {
    appendLink(document, {
      rel: "modulepreload",
      crossOrigin: "",
      href: file
    });
  } else if (file.endsWith(".css")) {
    appendLink(document, {
      rel: "stylesheet",
      href: file
    });
  }
}
var createLink = (document) => document.createElement("link");
var setAttrs = (el, attrs) => {
  const keys = Object.keys(attrs);
  for (const key of keys)
    el.setAttribute(key, attrs[key]);
};
function appendLink(document, attrs) {
  const exits = document.head.querySelector(`link[href='${attrs.file}']`);
  if (exits)
    return;
  const link = createLink(document);
  setAttrs(link, attrs);
  document.head.appendChild(link);
}

// src/node/renderWorker.ts
var renderWorker_default = async ({ route, dirStyle, outDir, indexHTML, formatting, out, ssrEntryPath, ssrManifest }) => {
  try {
    const { createApp } = await Promise.resolve().then(() => _interopRequireWildcard(require(ssrEntryPath)));
    const relativeRouteFile = `${(route.endsWith("/") ? `${route}index` : route).replace(/^\//g, "")}.html`;
    const filename = dirStyle === "nested" ? _path.join.call(void 0, route.replace(/^\//g, ""), "index.html") : relativeRouteFile;
    console.log(
      `Page ${_chalk2.default.dim(`${outDir}/`)}${_chalk2.default.cyan(filename.padEnd(15, " "))}`
    );
    const appCtx = await createApp(false, route);
    const { app, router, head, initialState } = appCtx;
    if (router) {
      await router.push(route);
      await router.isReady();
    }
    const transformedIndexHTML = (
      /* (await onBeforePageRender?.(route, indexHTML, appCtx)) || */
      indexHTML
    );
    const ctx = {};
    const appHTML = await _serverrenderer.renderToString.call(void 0, app, ctx);
    const renderedHTML = renderHTML({ indexHTML: transformedIndexHTML, appHTML, initialState });
    const jsdom = new (0, _jsdom.JSDOM)(renderedHTML);
    renderPreloadLinks(jsdom.window.document, ctx.modules || /* @__PURE__ */ new Set(), ssrManifest);
    head == null ? void 0 : head.updateDOM(jsdom.window.document);
    const html = jsdom.serialize();
    const transformed = (
      /* (await onPageRendered?.(route, html, appCtx)) || */
      html
    );
    const formatted = await format(transformed, formatting);
    await _fsextra2.default.ensureDir(_path.join.call(void 0, out, _path.dirname.call(void 0, filename)));
    await _fsextra2.default.writeFile(_path.join.call(void 0, out, filename), formatted, "utf-8");
    delete app.config.errorHandler;
    delete app.config.globalProperties;
    delete app._context.provides;
    delete app._context.components;
    delete app._context.mixins;
    delete app._context.config;
    delete app._context;
    delete app._component;
  } catch (err) {
    console.error(`${_chalk2.default.gray("[vite-ssg]")} ${_chalk2.default.red(`Error on page: ${_chalk2.default.cyan(route)}`)}
${err.stack}`);
  }
};
function renderHTML({ indexHTML, appHTML, initialState }) {
  const stateScript = initialState ? `
<script>window.__INITIAL_STATE__=${initialState}</script>` : "";
  return indexHTML.replace(
    '<div id="app"></div>',
    `<div id="app" data-server-rendered="true">${appHTML}</div>${stateScript}`
  );
}
async function format(html, formatting) {
  if (formatting === "minify") {
    const htmlMinifier = await Promise.resolve().then(() => _interopRequireWildcard(require("html-minifier")));
    return htmlMinifier.minify(html, {
      collapseWhitespace: true,
      caseSensitive: true,
      collapseInlineTagWhitespace: false,
      minifyJS: true,
      minifyCSS: true
    });
  } else if (formatting === "prettify") {
    const prettier = (await Promise.resolve().then(() => _interopRequireWildcard(require("prettier/esm/standalone.mjs")))).default;
    const parserHTML = (await Promise.resolve().then(() => _interopRequireWildcard(require("prettier/esm/parser-html.mjs")))).default;
    return prettier.format(html, { semi: false, parser: "html", plugins: [parserHTML] });
  }
  return html;
}


exports.default = renderWorker_default;
